import org.uddi4j.*;
import org.uddi4j.response.*;
import org.uddi4j.client.*;
import org.uddi4j.datatype.*;
import org.uddi4j.datatype.business.*;
import org.uddi4j.datatype.service.*;
import org.uddi4j.datatype.tmodel.*;
import org.uddi4j.util.*;
import org.uddi4j.transport.*;

import java.util.*;
import java.net.*;

/**
 * This sample retrieves an existing businessEntity entry from a UDDI registry.
 * It is assumed that this entry eists, with a business name of "Bill's Office
 * Services". You can create this entry using the UDDI4JPublishSample class.
 * 
 * @author Andre Tost
 */
public class UDDI4JBusinessSample {

  public static final String inquiryURL = 
    "http://www-3.ibm.com/services/uddi/v2beta/inquiryapi";

  protected UDDIProxy proxy = null;
  protected BusinessEntity businessEntity = null;

  public static void main(String args[]) throws Exception {
    UDDI4JBusinessSample sample = new UDDI4JBusinessSample();
    try {
      sample.init();
      sample.findBusiness();
      sample.displayBusinessInfo();
    } catch (Exception e) {
      System.out.println("Exception : " + e);
      e.printStackTrace();
    }
  }

  // creates a new proxy instance for the specified access URLs
  protected void init() 
          throws MalformedURLException, UDDIException, TransportException {
    proxy = new UDDIProxy(new URL(inquiryURL), null);
  }

  // finds a specific business entry by its name
  protected void findBusiness() throws UDDIException, TransportException {

    // create a Vector of Name objects, which is used for the inquiry
    Vector names = new Vector();
    names.addElement(new Name("Bill's Office Services"));

    // make the call on the proxy
    BusinessList list = proxy.find_business(names, null, null, null, null, 
                                            null, 0);
    BusinessInfos infos = list.getBusinessInfos();

    // if the returned vector is empty, no business could be found
    if (infos.getBusinessInfoVector().size() == 0) {
      throw new RuntimeException("No business found!");
    }

    // here we simply assume that we only got one entry back with that name
    // obtain the unique identifier for this entry in the registry
    BusinessInfo info = 
      (BusinessInfo) infos.getBusinessInfoVector().elementAt(0);
    String businessKey = info.getBusinessKey();

    // use the unique identifier to obtain the full record
    businessEntity = 
      (BusinessEntity) proxy.get_businessDetail(businessKey)
        .getBusinessEntityVector().elementAt(0);
  }


  /**
   * displays detailed information about a specific businessEntity entry
   * in the registry
   */
  protected void displayBusinessInfo() {

    // print out some information about the entry
    System.out.println("Business Key : " + businessEntity.getBusinessKey());
    System.out.println("Business Name : " 
                       + businessEntity.getDefaultNameString());

    // see if it contains a categoryBag
    CategoryBag cb = businessEntity.getCategoryBag();
    if (cb != null) {

      // a categoryBag contains a vector of keyedReference elements
      // loop over each one
      Vector keyedReferences = cb.getKeyedReferenceVector();
      Enumeration enum = keyedReferences.elements();
      while (enum.hasMoreElements()) {
        KeyedReference kr = (KeyedReference) enum.nextElement();

        // each keyed Reference element contains three values
        // namely the key name and value, plus the tModel key
        System.out.println("\tKey name : " + kr.getKeyName());
        System.out.println("\tKey Value : " + kr.getKeyValue());
        System.out.println("\tTModel Ref ID : " + kr.getTModelKey());
      }
    }

    System.out.println();

    // now find out about the contained service entries
    // again, all service entries are contained in a vector
    BusinessServices bs = businessEntity.getBusinessServices();
    if (bs != null) {
      Vector services = bs.getBusinessServiceVector();
      Enumeration enum = services.elements();
      while (enum.hasMoreElements()) {

        // print out each service's name and key
        BusinessService service = (BusinessService) enum.nextElement();
        System.out.println("\tService Name : " 
                           + service.getDefaultNameString());
        System.out.println("\tService Key : " + service.getServiceKey());
      }
    }
  }
}