import org.uddi4j.*;
import org.uddi4j.response.*;
import org.uddi4j.client.*;
import org.uddi4j.datatype.*;
import org.uddi4j.datatype.service.*;
import org.uddi4j.datatype.binding.*;
import org.uddi4j.datatype.tmodel.*;
import org.uddi4j.util.*;
import org.uddi4j.transport.*;

import java.util.*;
import java.net.*;

/**
 * This example shows how you can add information about a service described by a WSDL document
 * to an existing businessEntity entry in a UDDI registry. It creates a new tModel,
 * then creates a new businessService and bindingTemplate.
 * 
 * The example expects an existing businessEntity element, with the name of the business
 * being "Bill's Office Services". You can use the UDDI4JPublishSample class to 
 * create such an entry.
 * 
 * @author Andre Tost
 */
public class UDDI4JCompleteSample {

  // the following Strings point to the current IBM version 2 beta registry
  // the URLs may change over time as the version 2 registry is rolled into
  // production
  public static final String inquiryURL = 
    "http://www-3.ibm.com/services/uddi/v2beta/inquiryapi";
  public static final String publishURL = 
    "https://www-3.ibm.com/services/uddi/v2beta/protect/publishapi";
  public static final String userid = "[userid]";
  public static final String password = "[password]";

  protected UDDIProxy proxy = null;
  protected String authInfo = null;
  protected String tModelKey = null;
  protected String businessKey = null;
  protected String serviceKey = null;
  protected String bindingKey = null;

  public static void main(String args[]) throws Exception {

    // the following code enables the client to create a HTTPS connection URL
    // this is not necessary if you run this code on a JDK version 1.4
    System.setProperty("java.protocol.handler.pkgs", 
                       "com.sun.net.ssl.internal.www.protocol");
    java.security.Security
      .addProvider(new com.sun.net.ssl.internal.ssl.Provider());
    UDDI4JCompleteSample sample = new UDDI4JCompleteSample();
    try {
      sample.init();
      sample.saveTModel();
      sample.findBusiness();
      sample.addService();
      sample.addBindingTemplate();
    } catch (Exception e) {
      System.out.println("Exception : " + e);
    }
  }

  // creates a new proxy instance for the specified access URLs
  protected void init() 
          throws MalformedURLException, UDDIException, TransportException {

    proxy = new UDDIProxy(new URL(inquiryURL), new URL(publishURL));

    // Get an authorization token
    // Pass in userid and password registered at the UDDI site
    authInfo = proxy.get_authToken(userid, password).getAuthInfoString();
  }

  /**
   * creates a new tModel, representing a web service WSDL document, and saves it in
   * the registry
   */
  protected void saveTModel() throws UDDIException, TransportException {

    TModel tm = new TModel();
    tm.setName("Addressbook tModel");

    // need to set empty key, this indicates to the
    // registry that we are creating a new entry
    tm.setTModelKey("");

    // the URL of the WSDL interface definition goes into the
    // overviewDoc / overviewURL element of the tModel
    OverviewDoc od = new OverviewDoc();
    od.setOverviewURL("http://localhost:8080/Wrox/wsdl/Addressbook-service.wsdl");
    tm.setOverviewDoc(od);

    // the first keyedReference points ot the WSDL spec to indicate
    // that this tModel refers to a WSDL document
    KeyedReference kr1 = new KeyedReference("uddi-org:types", "wsdlSpec");
    kr1.setTModelKey("UUID:C1ACF26D-9672-4404-9D70-39B756E62AB4");

    // the second keyedReference indicates the type of service that
    // is offered, by UNSPSC code
    KeyedReference kr2 = 
      new KeyedReference("Office administration or secretarial services", 
                         "80161501");
    kr2.setTModelKey("UUID:DB77450D-9FA8-45D4-A7BC-04411D14E384");

    Vector krv = new Vector();
    krv.addElement(kr1);
    krv.addElement(kr2);

    CategoryBag cb = new CategoryBag();
    cb.setKeyedReferenceVector(krv);

    tm.setCategoryBag(cb);

    // Save tModel
    Vector entities = new Vector();
    entities.addElement(tm);
    TModelDetail r = proxy.save_tModel(authInfo, entities);

    tModelKey = ((TModel) r.getTModelVector().elementAt(0)).getTModelKey();
    System.out.println("Created tModel with key " + tModelKey);
    System.out.println("tmodel is " + tm.toString());
  }

  /**
   * retrieves the key of a particular businessEntity element the name of this
   * business must be "Bill's Office Services". The samples assumes that this entry
   * already exists. You can use the UDDI4JPublishSample class to create this entry.
   */
  protected void findBusiness() throws UDDIException, TransportException {
    Vector names = new Vector();
    names.addElement(new Name("Bill's Office Services"));
    BusinessList bl = proxy.find_business(names, null, null, null, null, 
                                          null, 0);
    Vector biVector = bl.getBusinessInfos().getBusinessInfoVector();
    if (biVector.size() == 0) {
      throw new RuntimeException("Could not find business!!");
    }

    // we simply assume that only one entry with that name exists
    businessKey = ((BusinessInfo) biVector.elementAt(0)).getBusinessKey();
    System.out.println("Found business with key " + businessKey);
  } 

  /**
   * creates a new businessService element and associates it with the businessEntity
   * element retrieved earlier.
   * Also adds a categoryBag to the new entry
   */
  protected void addService() throws UDDIException, TransportException {

    BusinessService bs = new BusinessService();
    bs.setDefaultNameString("Addressbook Service", "en");

    // the business key was retrieved earlier
    bs.setBusinessKey(businessKey);

    // must set to empty to indicate new entry
    bs.setServiceKey("");

    // must add vector with bindingTemplates, even though it is empty here
    Vector btv = new Vector();
    BindingTemplates bts = new BindingTemplates();
    bts.setBindingTemplateVector(btv);
    bs.setBindingTemplates(bts);

    KeyedReference kr = 
      new KeyedReference("Office administration or secretarial services", 
                         "80161501");
    kr.setTModelKey("UUID:DB77450D-9FA8-45D4-A7BC-04411D14E384");

    Vector krv = new Vector();
    krv.addElement(kr);

    CategoryBag cb = new CategoryBag();
    cb.setKeyedReferenceVector(krv);

    bs.setCategoryBag(cb);

    // Save service
    Vector entities = new Vector();
    entities.addElement(bs);
    ServiceDetail r = proxy.save_service(authInfo, entities);

    serviceKey = 
      ((BusinessService) r.getBusinessServiceVector().elementAt(0))
        .getServiceKey();
    System.out.println("Create service with key " + serviceKey);
  }

  /**
   * creates a new bindingTemplate element its instanceDetails point to the WSDL
   * tModel created earlier access point is the SOAP rpcrouter servlet
   */
  protected void addBindingTemplate() 
          throws UDDIException, TransportException {

    BindingTemplate bt = new BindingTemplate();

    // associate with businessService entry created earlier
    bt.setServiceKey(serviceKey);

    // must set to empty to indicate new entry
    bt.setBindingKey("");

    // the access point indicates where this service can
    // be accessed. in this case, since we have a web service
    // hosted by an Apache SOAP environment, the access point
    // refers to an installed Apache SOAP environment
    AccessPoint ap = new AccessPoint();
    ap.setURLType("http");
    ap.setText("http://localhost:8080/soap/servlet/rpcrouter");

    bt.setAccessPoint(ap);

    // this overviewDoc points to the implementation part of
    // the WSDL document - is then added to the instance details
    OverviewDoc tmod = new OverviewDoc();
    tmod
      .setOverviewURL("http://localhost:8080/Wrox/wsdl/AddressbookService.wsdl");

    InstanceDetails instanceDetails = new InstanceDetails();
    instanceDetails.setOverviewDoc(tmod);

    // the tModelInstanceInfo contains the instance details and is associated
    // with the the tModel that we created above through its key
    TModelInstanceInfo tmii = new TModelInstanceInfo(tModelKey);
    tmii.setInstanceDetails(instanceDetails);

    Vector tmiiv = new Vector();
    tmiiv.addElement(tmii);

    TModelInstanceDetails tmid = new TModelInstanceDetails();
    tmid.setTModelInstanceInfoVector(tmiiv);

    bt.setTModelInstanceDetails(tmid);

    Vector entities = new Vector();
    entities.addElement(bt);
    BindingDetail r = proxy.save_binding(authInfo, entities);

    bindingKey = 
      ((BindingTemplate) r.getBindingTemplateVector().elementAt(0))
        .getBindingKey();
    System.out.println("Created bindingTemplate with key " + bindingKey);
  }
}